// vim: set tabstop=2 shiftwidth=2:

/**
 * @fileoverview SeaJS Package Manager Registry Apps.
 * @author yyfrankyy@gmail.com (Frank Xu)
 */

var express = require('express'),
    mustache = require('mustache'),
    fs = require('fs'),
    path = require('path'),
    config = require('./config'),
    app = express.createServer();
app.use('/assets', express.static(__dirname + '/assets'));
app.use('/modules', express.static(__dirname + '/modules'));

app.get('/', function(req, res) {
  fs.readFile(path.join(config.ROOT, 'templates', 'index.mustache'),
      function(err, data) {
        res.send(data.toString());
      });
});

app.get('/server', function(req, res) {
  var o = '<h1>cloudfoundry.com</h1>';
  o += '<h2>process.version: ' + process.version + '</h2>';
  o += '<pre>';
  for (var i in process.env) {
    o += i + ':' + process.env[i] + '\n';
  }
  o += '</pre>';
  o += '<h2>npm.version: ' + require('npm').version + '</h2>';
  res.send(o, {'Content-Type': 'text/html'});
});

app.get('/registry', function(req, res) {
  fs.readFile(path.join(config.ROOT, 'registry'),
      function(err, data) {
        jsonp_wrap(req, res, JSON.parse(data));
      });
});

app.get('/search/:query', function(req, res) {
  var data,
      query = req.params.query.toLowerCase(),
      results = {},
      meta = {};
  try {
    data = fs.readFileSync(path.join(config.ROOT, 'registry')).toString();
    data = JSON.parse(data);
  } catch (e) {
    console.error(e.message);
    data = {};
  }

  for (var k in data) {
    meta = data[k];
    for (var i in meta) {
      if (typeof meta[i] === 'array') {
        if (meta[i].join().indexOf(query) !== -1) {
          results[meta.name] = meta;
          break;
        }
      } else {
        if (meta[i].toString().toLowerCase().indexOf(query) !== -1) {
          results[meta.name] = meta;
          break;
        }
      }
    }
  }

  jsonp_wrap(req, res, results);
});
app.listen(process.env.VMC_APP_PORT || 3000);

function jsonp_wrap(req, res, results) {
  var callback = req.query.callback, output;
  if (callback && !/^[A-Za-z_.]+$/.test(callback)) {
    results = {message: 'Invalid Callback. ' + callback};
    res.send(JSON.stringify(results));
    return;
  }
  output = JSON.stringify(results);
  res.send(callback ? callback + '(' + output + ');' : output);
}
